# Generated by Django 2.2.16 on 2021-03-05 14:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0040_page_draft_title'),
        ('wagtailsharing', '0002_shareableroutablepage'),
    ]

    operations = [
        migrations.DeleteModel(
            name='ShareableRoutablePage',
        ),
    ]
