# Generated by Django 2.2.17 on 2021-02-22 12:17

from django.db import migrations, models
import django.db.models.deletion
import wagtail.contrib.routable_page.models
import wagtailsharing.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0041_group_collection_permissions_verbose_name_plural'),
    ]

    operations = [
        migrations.CreateModel(
            name='RoutableTestPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('text', models.TextField()),
            ],
            options={
                'abstract': False,
            },
            bases=(wagtail.contrib.routable_page.models.RoutablePageMixin, 'wagtailcore.page'),
        ),
        migrations.CreateModel(
            name='ShareableRoutableTestPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('text', models.TextField()),
            ],
            options={
                'abstract': False,
            },
            bases=(wagtailsharing.models.ShareableRoutablePageMixin, 'wagtailcore.page'),
        ),
    ]
